/** \file Main.h
    \brief Application main form header file
*/

#ifndef MainH
#define MainH
//---------------------------------------------------------------------------
#include "display/Display.h"
#include "Mutex.h"

#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include <ExtCtrls.hpp>
#include <ComCtrls.hpp>
#include <Menus.hpp>
#include <ToolWin.hpp>
#include <ImgList.hpp>
#include <ActnList.hpp>

class DeviceInterface;
struct S_SCOPE_CAPABILITIES;

//---------------------------------------------------------------------------
/** \brief Main application window
*/
class TfrmMain : public TForm
{
__published:	// IDE-managed Components
	TPanel *pnlRight;
	TPanel *pnlCenter;
	TTimer *Timer;
	TStatusBar *StatusBar;
	TMainMenu *Menu;
	TMenuItem *File1;
	TMenuItem *miHelp;
	TMenuItem *E1;
	TMenuItem *miSettings;
	TImageList *imglToolbar;
	TCoolBar *CoolBar;
	TToolBar *ToolBarMode;
	TToolButton *btnModeScope;
	TToolButton *btnModeSpectrum;
	TToolButton *btnModeRecorder;
	TToolBar *ToolBarMouse;
	TToolButton *btnMouseCursor;
	TToolButton *btnMouseZoomXY;
	TToolButton *btnMouseMove;
	TActionList *ActionList;
	TAction *ActionChangeMouseModeCursor;
	TAction *ActionChangeMouseModeOffset;
	TAction *ActionChangeMouseModeZoom;
	TToolButton *btnMouseZoomX;
	TToolButton *btnMouseZoomY;
	TAction *ActionChangeMouseModeZoomX;
	TAction *ActionChangeMouseModeZoomY;
	TToolButton *btnZoomClear;
	TAction *ActionZoomClear;
	TAction *ActionModeScope;
	TAction *ActionModeSpectrum;
	TMenuItem *miAbout;
	TMenuItem *miCommonSettings;
	TMenuItem *miDeviceSettings;
	TMenuItem *miView;
	TMenuItem *miViewLog;
	TAction *ActionAcquireStart;
	TAction *ActionAcquireStop;
	TTimer *tmrUpdateConnState;
	TTrackBar *trbarSensitivity;
	TLabel *lblSensitivity;
	TLabel *lblSensitivityVal;
	TLabel *lblTriggerType;
	TComboBox *cbTriggerType;
	TButton *btnManualTrigger;
	TComboBox *cbBufferSize;
	TLabel *lblBuffer;
	TLabel *lblSampling;
	TLabel *lblSamplingVal;
	TTrackBar *trbarSampling;
	TTrackBar *trbarTriggerLevel;
	TLabel *lblTriggerLevel;
	TLabel *lblTriggerLevelVal;
	TLabel *lblCouplingType;
	TComboBox *cbCouplingType;
	TButton *btnStart;
	TButton *btnStop;
	TPaintBox *PaintBoxCenter;
	void __fastcall pnlCenterResize(TObject *Sender);
	void __fastcall TimerTimer(TObject *Sender);
	void __fastcall pnlCenterMouseDown(TObject *Sender, TMouseButton Button,
          TShiftState Shift, int X, int Y);
	void __fastcall ActionChangeMouseModeCursorExecute(TObject *Sender);
	void __fastcall ActionChangeMouseModeOffsetExecute(TObject *Sender);
	void __fastcall ActionChangeMouseModeZoomExecute(TObject *Sender);
	void __fastcall ActionChangeMouseModeZoomXExecute(TObject *Sender);
	void __fastcall ActionChangeMouseModeZoomYExecute(TObject *Sender);
	void __fastcall ActionZoomClearExecute(TObject *Sender);
	void __fastcall pnlCenterMouseUp(TObject *Sender, TMouseButton Button,
		  TShiftState Shift, int X, int Y);
	void __fastcall ActionModeScopeExecute(TObject *Sender);
	void __fastcall ActionModeSpectrumExecute(TObject *Sender);
	void __fastcall miAboutClick(TObject *Sender);
	void __fastcall miCommonSettingsClick(TObject *Sender);
	void __fastcall FormCreate(TObject *Sender);
	void __fastcall miDeviceSettingsClick(TObject *Sender);
	void __fastcall miViewLogClick(TObject *Sender);
	void __fastcall ActionAcquireStartExecute(TObject *Sender);
	void __fastcall ActionAcquireStopExecute(TObject *Sender);
	void __fastcall tmrUpdateConnStateTimer(TObject *Sender);
	void __fastcall cbTriggerTypeChange(TObject *Sender);
	void __fastcall btnManualTriggerClick(TObject *Sender);
	void __fastcall cbBufferSizeChange(TObject *Sender);
	void __fastcall cbCouplingTypeChange(TObject *Sender);
	void __fastcall PaintBoxCenterPaint(TObject *Sender);
	void __fastcall PaintBoxCenterMouseDown(TObject *Sender, TMouseButton Button,
          TShiftState Shift, int X, int Y);
	void __fastcall PaintBoxCenterMouseUp(TObject *Sender, TMouseButton Button,
          TShiftState Shift, int X, int Y);
	void __fastcall FormCloseQuery(TObject *Sender, bool &CanClose);
private:	// User declarations
	/** \brief Pointer to plot graph object
	*/
	Display *display;
	/** \brief Type of tool to use with plot graph
	*/
	enum E_MOUSE_TOOL { MOUSE_CURSOR = 0,
						MOUSE_OFFSET,
						MOUSE_ZOOM,
						MOUSE_ZOOM_X,
						MOUSE_ZOOM_Y } mouse_tool;
    /** \brief Resources ID's used as cursors for plot graph
    */
	enum E_CURSOR_RES { CURSOR_HAND = 100,
						CURSOR_ZOOM };
    /** \brief Current plot state (depending on mode)
    */
	struct DISP_STATE {
		float fZoomX;
		float fZoomY;
	} displayOsc, displaySpe;
	/** \brief Get structure with current display attributes
	*/
	inline struct DISP_STATE& DisplayState(void) {
		if (display->GetMode() == MODE_OSCILLOSCOPE) {
			return displayOsc;
		} else {
			return displaySpe;
		}
	}
	/** \brief Plot zoom multiplier for every click with zoom tool
	*/
	const float fZoomChangeGrid;
	void UpdateCursorReadings(enum E_DISPLAY_MODE, int cursorId, float fValX, float fValY);
	void CalculateSpectrum(void);
	int iPlotMoveStartX, iPlotMoveStartY;	///< temporary variables used to pass offset vector to display
	bool bMovingPlot;                       ///< are we moving plot right now?
	bool bDataUpdate;						///< flag set when new data is acquired
	bool bPaintUpdate;                      ///< flag set when plot canvas has to be refreshed
	std::vector<float> data;                ///< data container used in oscilloscope and slow signal recorder modes
	std::vector<float> dataFFT;             ///< data container used in spectrum analyzer mode
    AnsiString asDevDllDir;					///< device dll's directory
	DeviceInterface *device;                ///< pointer to object that interfaces with data acquisision hardware
	Mutex mutexDeviceData;                  ///< mutex that prevents hazards with simultaneous data updating by device and data plotting
	void UpdateConnectionState(void);
	/** \brief Update user interface to reflect settings types and ranges offered by device
        \param capabilities ptr to structure describing device properties
	*/
	void SetCapabilities(const struct S_SCOPE_CAPABILITIES* const capabilities);
	/** \brief Change device trigger type and update GUI when trigger type is changed by user
	*/
	void UpdateTriggerType(void);
	/** \brief Change device coupling (AC/DC/...) type and update GUI when coupling type is changed by user
	*/
	void UpdateCouplingType(void);
	/** \brief Change size of device acquisition buffer
	*/
	void UpdateBufferSize(void);
	/** \brief Change device sensitivity/gain
	*/
	void UpdateSensitivity(void);
	/** \brief Change device sampling rate
	*/
	void UpdateSampling(void);
	/** \brief Update device triggering voltage level
	*/
    void UpdateTriggerLevel(void);

	void LoadDevice(AnsiString dllname);

	void __fastcall TrackBarMouseUp(TObject *Sender, TMouseButton Button, TShiftState Shift, int X, int Y);

public:		// User declarations
	__fastcall TfrmMain(TComponent* Owner);
	__fastcall ~TfrmMain();
	/** \brief Called when log window is closed
	*/
	void CallbackCloseLogFn(void);
	/** \brief Called by device when new data is available
	*/
	void CallbackDataFn(short*, int);
	/** \brief Called by device when connection state is changed
        \param state current connection state
        \param szMsgText additional text message related to current connection state
	*/
	void CallbackConnectFn(int state, char *szMsgText);
	/** \brief Called by device when triggered
	*/
	void CallbackTriggerFn(void);
};

//---------------------------------------------------------------------------
extern PACKAGE TfrmMain *frmMain;
//---------------------------------------------------------------------------
#endif
